import logging
import os
import threading
import time
from logging.handlers import TimedRotatingFileHandler

from pesto_common.config.configer import Configer


def doRollover(self):
    """
    do a rollover; in this case, a date/time stamp is appended to the filename
    when the rollover happens.  However, you want the file to be named for the
    start of the interval, not the current time.  If there is a backup count,
    then we have to get a list of matching filenames, sort them and remove
    the one with the oldest suffix.
    """
    if self.stream:
        self.stream.close()
        self.stream = None
    # get the time that this sequence started at and make it a TimeTuple
    currentTime = int(time.time())
    dstNow = time.localtime(currentTime)[-1]
    t = self.rolloverAt - self.interval
    if self.utc:
        timeTuple = time.gmtime(t)
    else:
        timeTuple = time.localtime(t)
        dstThen = timeTuple[-1]
        if dstNow != dstThen:
            if dstNow:
                addend = 3600
            else:
                addend = -3600
            timeTuple = time.localtime(t + addend)
    dfn = self.baseFilename.replace('.log', '') + '.' + time.strftime(self.suffix, timeTuple) + '.log'
    if os.path.exists(dfn):
        os.remove(dfn)
    # Issue 18940: A file may not have been created if delay is True.
    if os.path.exists(self.baseFilename):
        os.rename(self.baseFilename, dfn)
    if self.backupCount > 0:
        for s in self.getFilesToDelete():
            os.remove(s)
    if not self.delay:
        self.stream = self._open()
    newRolloverAt = self.computeRollover(currentTime)
    while newRolloverAt <= currentTime:
        newRolloverAt = newRolloverAt + self.interval
    # If DST changes and midnight or weekly rollover, adjust for this.
    if (self.when == 'MIDNIGHT' or self.when.startswith('W')) and not self.utc:
        dstAtRollover = time.localtime(newRolloverAt)[-1]
        if dstNow != dstAtRollover:
            if not dstNow:  # DST kicks in before next rollover, so we need to deduct an hour
                addend = -3600
            else:  # DST bows out before next rollover, so we need to add an hour
                addend = 3600
            newRolloverAt += addend
    self.rolloverAt = newRolloverAt


logging.handlers.TimedRotatingFileHandler.doRollover = doRollover


def emit(self, record):
    try:
        msg = self.format(record)
        stream = self.stream
        fs = "%s\n"
        try:
            stream.write(fs % msg)
        except UnicodeError:
            try:
                stream.write(fs % msg.encode("UTF-8"))
            except UnicodeDecodeError:
                stream.write(fs % msg.decode('ISO-8859-1'))
        self.flush()
    except (KeyboardInterrupt, SystemExit):
        raise
    except:
        self.handleError(record)


logging.StreamHandler.emit = emit

lock = threading.Lock()

global loggerFactory
loggerFactory = None  # type: LoggerFactory


class LoggerFactory(object):

    def __init__(self, filename, level=logging.INFO, backup_count=7):
        """
        :param filename:
        :param loggername:
        """
        # self.filename = filename.format(str(time.strftime("%Y-%m-%d", time.localtime())))
        self.filename = filename
        self.level = level
        self.backup_count = backup_count
        self.suffix = '.log'
        self.prefix = self.filename.replace(self.suffix, '')

    def __get_logger(self, logger_name=None, level=None, format="%(asctime)s [%(threadName)s] %(levelname)s  %(name)s - %(message)s"):
        # logging.basicConfig(level=level, format=format)

        if level is None:
            level = self.level

        if not logging.root.handlers:
            # file_handler = logging.FileHandler(self.filename, 'a+')
            # file_handler.setFormatter(logging.Formatter(format))
            # logging.root.addHandler(file_handler)
            logging.root.setLevel(self.level)

            formatter = logging.Formatter(format)

            console_handler = logging.StreamHandler()
            console_handler.setFormatter(formatter)
            console_handler.encoding = 'utf-8'
            logging.root.addHandler(console_handler)

            file_handler = TimedRotatingFileHandler(filename=self.filename, when='midnight', interval=1, backupCount=self.backup_count, encoding='utf-8')
            file_handler.setFormatter(formatter)
            file_handler.suffix = "%Y-%m-%d"
            file_handler.extMatch = r"^\d{4}-\d{2}-\d{2}$"
            logging.root.addHandler(file_handler)

        logger = logging.getLogger(logger_name)
        logger.level = level
        return logger

    def get_filename(self):
        return self.filename

    @staticmethod
    def get_logger(logger_name):
        LoggerFactory.__init_logger_factory()

        global loggerFactory
        logger = loggerFactory.__get_logger(logger_name=logger_name)
        return logger

    @staticmethod
    def __init_logger_factory():

        try:
            lock.acquire()
            global loggerFactory
            if loggerFactory is None:
                log_path = Configer.get('log.path')
                if log_path is None:
                    raise RuntimeError('log.path not found')
                log_dir = os.path.dirname(log_path)
                if not os.path.exists(log_dir):
                    os.makedirs(log_dir)

                log_level = Configer.get('log.level')
                if log_level is None:
                    raise RuntimeError('log.level not found')
                log_level = log_level.upper() if log_level is not None else ''
                if log_level == 'DEBUG':
                    level = logging.DEBUG
                elif log_level == 'INFO':
                    level = logging.INFO
                elif log_level == 'WARNING':
                    level = logging.WARNING
                elif log_level == 'WARN':
                    level = logging.WARN
                elif log_level == 'ERROR':
                    level = logging.ERROR
                else:
                    raise RuntimeError('log level error: {}'.format(log_level))

                log_backup_count = Configer.get('log.backup_count')
                if log_backup_count is None:
                    log_backup_count = 7
                loggerFactory = LoggerFactory(log_path, level, log_backup_count)

                logger = loggerFactory.__get_logger(logger_name='logger.factory')
                env = os.environ.get('env', 'local')
                logger.info('Start env: ' + env)
                logger.info('Log file path: {}, default level: {}'.format(loggerFactory.get_filename(), log_level))
        finally:
            lock.release()



#!/usr/bin/env python
# encoding: utf-8
from abc import abstractmethod, ABCMeta

from pesto_common.log.logger_factory import LoggerFactory
from pesto_orm.core.model import Model

logger = LoggerFactory.get_logger("pipeline.step")


class PipelineError(RuntimeError):
    pass


class PipelineStepData(Model):
    pass


class PipelineStep:
    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, data={}, next_step=None):
        if isinstance(data, PipelineStepData):
            self.data = data
        else:
            self.data = PipelineStepData(init_attrs=data)  # 执行需要的数据
        self.result = PipelineStepData()  # 执行的结果数据
        self.next_step = next_step  # 下一步执行内容
        self.index = 1
        self.exit = False

    def start(self):
        # 前置逻辑 如: 数据处理
        self._before()
        # 核心步骤
        self._run()
        # 后置逻辑 如: 释放资源
        self._after()
        self.__pass_data()
        if not self.exit:
            if self.next_step is not None:
                return self.next_step.start()
        return True

    def set_next_step(self, next_step):
        if self.next_step is None:
            next_step.index = self.index + 1
            self.next_step = next_step
        else:
            self.next_step.set_next_step(next_step)
        return self

    def __pass_data(self):
        if self.next_step is not None:
            for key in self.result.get_attrs():
                if self.next_step.data.contains_attr(key):
                    raise PipelineError('Next step contains key for data, duplicate key: ' + key)
                self.next_step.data.set_attr(key, self.result.get_attr(key))
            # clean data
            self.data.clear_attrs()
            self.result.clear_attrs()

    def _before(self):
        pass

    @abstractmethod
    def _run(self):
        raise NotImplementedError

    def _after(self):
        pass



# -*- coding:utf8 -*-
import socket
from contextlib import closing

from log.logger_factory import LoggerFactory

logger = LoggerFactory.get_logger('utils.ip_utils')


class IpUtils(object):
    def __init__(self):
        pass

    @staticmethod
    def get_host_ip():
        with closing(socket.socket(socket.AF_INET, socket.SOCK_DGRAM)) as s:
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]

        return ip



# -*- coding:utf8 -*-

class ReflectUtils(object):
    def __init__(self):
        pass

    @staticmethod
    def create_instance(module, class_name, *args, **kwargs):
        module_meta = __import__(module, globals(), locals(), [class_name])
        class_meta = getattr(module_meta, class_name)
        obj = class_meta(*args, **kwargs)
        return obj



# -*- coding:utf8 -*-
import hashlib


class HashUtils(object):
    def __init__(self):
        pass

    @staticmethod
    def md5(content):
        md5 = hashlib.md5()
        md5.update(content)
        return md5.hexdigest()



# encoding:utf-8
from collections import Callable, OrderedDict


class CollectionUtils(object):
    __caches = {}

    def __init__(self):
        pass

    @staticmethod
    def split(data=[], num=1):
        length = len(data)
        split_size = int(length / num)
        split_result = []
        if split_size * num == length:
            for i in range(0, length, split_size):
                split_result.append(data[i:i + split_size])
        else:
            over_size = length - num * split_size
            start = over_size * (split_size + 1)

            for i in range(0, start, split_size + 1):
                split_result.append(data[i:i + split_size + 1])

            for j in range(start, length, split_size):
                split_result.append(data[j:j + split_size])

        return split_result

    @staticmethod
    def drop_duplicates(data):
        """
        remove duplicates
        :param data:
        :return:
        """
        if len(data) > 1:
            data.sort()
            result = [data[0]]
            for i in range(1, len(data)):
                if data[i] != result[- 1]:
                    result.append(data[i])
            return result
        return data

    @staticmethod
    def difference(data1, data2):
        """
        element in data1 and not in data2
        :param data1:
        :param data2:
        :return:
        """
        l1 = len(data1)
        l2 = len(data2)
        if l1 > 0 and l2 > 0:
            data1.sort()
            data2.sort()

            result = []
            i1 = 0
            i2 = 0
            while i1 < l1 and i2 < l2:
                if data1[i1] < data2[i2]:
                    result.append(data1[i1])
                    i1 += 1
                elif data1[i1] > data2[i2]:
                    i2 += 1
                else:
                    i1 += 1

            if i1 < l1:
                result.extend(data1[i1:])

            return result
        return data1

    @staticmethod
    def search(data, x):
        """
        有序列表查找
        :param data:
        :param x:
        :return:
        """
        l = len(data)
        lo = 0
        hi = l
        while lo < hi:
            mid = (lo + hi) // 2
            if data[mid] < x:
                lo = mid + 1
            else:
                hi = mid
        return lo if lo < l and data[lo] == x else -1

    @staticmethod
    def insert(data, x):
        """
        有序列表插入
        :param data:
        :param x:
        :return:
        """
        lo = 0
        hi = len(data)
        while lo < hi:
            mid = (lo + hi) // 2
            if data[mid] < x:
                lo = mid + 1
            else:
                hi = mid
        data.insert(lo, x)
        return lo


class DefaultOrderedDict(OrderedDict):
    # Source: http://stackoverflow.com/a/6190500/562769
    def __init__(self, default_factory=None, *a, **kw):
        if (default_factory is not None and
                not isinstance(default_factory, Callable)):
            raise TypeError('first argument must be callable')
        OrderedDict.__init__(self, *a, **kw)
        self.default_factory = default_factory

    def __getitem__(self, key):
        try:
            return OrderedDict.__getitem__(self, key)
        except KeyError:
            return self.__missing__(key)

    def __missing__(self, key):
        if self.default_factory is None:
            raise KeyError(key)
        self[key] = value = self.default_factory()
        return value

    def __reduce__(self):
        if self.default_factory is None:
            args = tuple()
        else:
            args = self.default_factory,
        return type(self), args, None, None, self.items()

    def copy(self):
        return self.__copy__()

    def __copy__(self):
        return type(self)(self.default_factory, self)

    def __deepcopy__(self, memo):
        import copy
        return type(self)(self.default_factory,
                          copy.deepcopy(self.items()))

    def __repr__(self):
        return 'DefaultOrderedDict(%s, %s)' % (self.default_factory,
                                               OrderedDict.__repr__(self))



# -*- coding:utf8 -*-
import datetime


class DateUtils(object):
    def __init__(self):
        pass

    @staticmethod
    def gen_dates(start, days):
        day = datetime.timedelta(days=1)
        for i in range(days):
            yield start + day * i

    @staticmethod
    def get_dates(start=None, end=None):
        """
        获取日期列表
        :param start: 开始日期
        :param end: 结束日期
        :return:
        """
        if start is None:
            start = datetime.datetime.strptime("1970-01-01", "%Y-%m-%d")
        if end is None:
            end = datetime.datetime.now()
        if isinstance(start, str):
            start = datetime.datetime.strptime(start, "%Y-%m-%d")
        if isinstance(end, str):
            end = datetime.datetime.strptime(end, "%Y-%m-%d")

        day_count = (end - start).days
        dates = []
        if day_count > 0:
            for day in DateUtils.gen_dates(start, (end - start).days + 1):
                dates.append(day)
        else:
            dates.append(start)
        return dates

    @staticmethod
    def utc_to_beijing(utc_time, format='%Y-%m-%dT%H:%M:%S.%fZ'):
        if isinstance(utc_time, datetime.datetime):
            date = utc_time + datetime.timedelta(hours=8)
        elif isinstance(utc_time, str):
            date = datetime.datetime.strptime(utc_time, format) + datetime.timedelta(hours=8)
        else:
            raise RuntimeError('not support this type: {}'.format(type(utc_time)))
        return date


if __name__ == '__main__':
    dates = DateUtils.get_dates(start=datetime.datetime.strptime("2019-03-01", "%Y-%m-%d"),
                                end=datetime.datetime.strptime("2019-03-02", "%Y-%m-%d"))
    print(dates)



# -*- coding:utf8 -*-
import codecs
import os
import sys
import threading
from configparser import ConfigParser, NoOptionError

global root_path
root_path = None
global root_project
root_project = None
global default_config_path
default_config_path = None

lock = threading.Lock()


class Configer(object):
    __configs = {}
    __env = os.environ.get('env', 'local')

    @staticmethod
    def get_env():
        return Configer.__env

    """
    set default config path
    """

    @staticmethod
    def use_default(path):
        global default_config_path
        default_config_path = path
        return Configer.use(path)

    @staticmethod
    def use(path):
        try:
            lock.acquire()
            try:
                return Configer.__configs[path]
            except KeyError:
                _config = _Config(Configer.__env, path)
                Configer.__configs[path] = _config
                return _config
        finally:
            lock.release()

    @staticmethod
    def refresh(path):
        if path in Configer.__configs:
            Configer.use(path)

    @staticmethod
    def __init():
        global root_path
        global root_project
        global default_config_path
        if default_config_path is None or root_path is None or root_project is None:
            current_path = os.path.abspath(sys.path[0])
            Configer.__search_config(current_path)
            if default_config_path is None:
                raise RuntimeError(
                    "if use this method, must init with `Configer.use_default(path)` before import class.")

    """
    get from default path
    """

    @staticmethod
    def get(name, default_value=None):
        Configer.__init()

        config_value = Configer.use(default_config_path).get(name, default_value)
        global root_path
        global root_project
        config_value = Configer.__convert_value(config_value, root_path, root_project)
        return config_value

    @staticmethod
    def __convert_value(config_value, root_path, root_project):
        if config_value is not None:

            if isinstance(config_value, str) and config_value.startswith('$root'):
                config_value = config_value.replace('$root_project', root_project).replace('$root_path', root_path)
        return config_value

    @staticmethod
    def contains(prefix):
        Configer.__init()
        config_options = Configer.use(default_config_path).items()
        for name in config_options:
            if name.startswith(prefix):
                return True
        return False

    @staticmethod
    def list(prefix):
        items = []
        Configer.__init()
        config_options = Configer.use(default_config_path).items()
        for name in config_options:
            if name.startswith(prefix):
                items.append(name)
        return items

    @staticmethod
    def __search_config(current_path, level=1):
        global root_path
        global root_project
        global default_config_path
        file_names = os.listdir(current_path)
        if 'requirements.txt' in file_names:
            root_path = current_path
            root_project = os.path.basename(root_path)

            if default_config_path is None:
                config_path = current_path + '/config.ini'
                if os.path.exists(current_path):
                    default_config_path = config_path

        if (default_config_path is None or root_path is None) and level <= 10:
            Configer.__search_config(os.path.dirname(current_path), level=level + 1)


class _Config(object):

    def __init__(self, env, path):
        self.env = env
        self.parser = ConfigParser(os.environ)
        self.parser.read_file(codecs.open(path, 'r', 'utf8'))

    def get(self, name, default_value=None):
        try:
            return self.parser.get(self.env, name)
        except NoOptionError:
            return default_value

    def items(self):
        return self.parser.options(self.env)


app_key = Configer.get('app.key')
if app_key is None:
    raise RuntimeError('before use configer, please set app.key to init')



# -*- coding:utf8 -*-
import csv
import gzip
import os
import pickle
import tarfile


class FileUtils(object):
    def __init__(self):
        pass

    @staticmethod
    def save_dict(data, file_path):
        if len(data) > 0:
            field_names = data[0].keys()
            file_dir = os.path.dirname(file_path)
            if not os.path.exists(file_dir):
                os.makedirs(file_dir)

            with open(file_path, 'w', newline='', encoding='utf-8-sig') as _f:
                writer = csv.DictWriter(_f, fieldnames=field_names, dialect='excel')
                writer.writeheader()
                for row in data:
                    writer.writerow(row)

    @staticmethod
    def clean_file(clean_file_path):
        if os.path.exists(clean_file_path):
            with open(clean_file_path, 'r+') as _file:
                _file.truncate()

    '''
    add file
    '''

    @staticmethod
    def add_to_file(add_file_path, item):
        with open(add_file_path, 'ab') as _file:
            _file.write(item)

    '''
    read file
    '''

    @staticmethod
    def __read_file(read_file_path):
        if os.path.exists(read_file_path):
            with open(read_file_path, 'rb') as _file:
                for line in _file:
                    yield line
        else:
            raise IOError('the file [{}] is not exist!'.format(read_file_path))

    @staticmethod
    def read_file_lines(read_file_path):
        lines = FileUtils.__read_file(read_file_path)
        if getattr(lines, '__iter__', None):
            return lines
        else:
            raise IOError('unknown error for read lines.')

    '''
    read gz file
    '''

    @staticmethod
    def __read_gz_file(read_file_path):
        if os.path.exists(read_file_path):
            with gzip.open(read_file_path, 'rb') as _file:
                for line in _file:
                    yield line
        else:
            raise IOError('the file [{}] is not exist!'.format(read_file_path))

    @staticmethod
    def read_gz_file_lines(read_file_path):
        lines = FileUtils.__read_gz_file(read_file_path)
        if getattr(lines, '__iter__', None):
            return lines
        else:
            raise IOError('unknown error for read lines.')

    @staticmethod
    def pack_gz(filename, source_dir):
        with tarfile.open(filename, "w:gz") as tar:
            tar.add(source_dir, arcname=os.path.basename(source_dir))

    @staticmethod
    def unpack_gz(filename, dest_dir):
        with tarfile.open(filename, "r:gz") as tar:
            tar.extractall(path=dest_dir)

    '''
    add pickel
    '''

    @staticmethod
    def add_to_pickle(add_file_path, item):
        with open(add_file_path, 'ab') as _file:
            pickle.dump(item, _file, pickle.HIGHEST_PROTOCOL)

    '''
    read pickel
    '''

    @staticmethod
    def read_pickle(read_file_path):
        if os.path.exists(read_file_path):
            with open(read_file_path, 'rb') as _file:
                try:
                    while True:
                        yield pickle.load(_file)
                except EOFError:
                    pass
        else:
            raise IOError('the file [{}] is not exist!'.format(read_file_path))

    @staticmethod
    def read_pickle_lines(read_file_path):
        lines = FileUtils.read_pickle(read_file_path)
        if getattr(lines, '__iter__', None):
            return lines
        else:
            raise IOError('unknown error for read lines.')


def main():
    FileUtils.clean_file('/Users/dreampie/PycharmProjects/dialog-mining/data/reply/standard_replies_1.txt')


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8

from setuptools import setup, find_packages

requirements = []

setup(name='pesto-common',
      version='0.0.6',
      author='Dreampie',
      author_email='Dreampie@outlook.com',
      url='https://github.com/Dreampie/pesto',
      description='Minimalist python utils',
      long_description='Minimalist python utils',
      long_description_content_type='text/markdown',
      keywords=['minimalist', 'python', 'utils', 'pesto'],
      packages=find_packages(),
      include_package_data=True,
      install_requires=requirements,
      platforms=['all']
      )



# -*- coding:utf8 -*-

class NumberUtils(object):
    def __init__(self):
        pass

    @staticmethod
    def is_number(s):
        try:
            float(s)
            return True
        except (TypeError, ValueError):
            pass

        try:
            import unicodedata
            unicodedata.numeric(s)
            return True
        except (TypeError, ValueError):
            pass

        return False
